/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package event;

import java.util.HashMap;
import java.util.Map;
import logging.Logger;
import main.GameApp;

/**
 * Basisklasse fuer jedes Event.
 * @author nico3000
 */
public abstract class EventData {
    private static final Map<Integer, Class<?>> knownEventDatas = new HashMap<>();
    
    @SuppressWarnings("OverridableMethodCallInConstructor")
    public EventData() {
        if(GameApp.isInDebugMode()) {
            Class<?> myClass = (Class<?>)this.getClass();
            int id = this.getId();
            Class<?> connectedClass = knownEventDatas.get(id);
            if(connectedClass == null) {
                knownEventDatas.put(id, myClass);
            } else if(connectedClass != myClass) {
                Logger.INSTANCE.error("Events " + myClass + " and " + connectedClass + " have the same ID!");
            }
        }
    }
    
    /**
     * Zeitstempel des Events.
     */
    long timeStamp = 0;
    
    /**
     * Liefert die ID des Events.
     * @return Die ID des Events
     */
    public abstract int getId();

    /**
     * Liefert den Zeitstempel des Events.
     * @return Der Zeritstempel
     */
    public long getTimeStamp() {
        return this.timeStamp;
    }
}
