/*
 * Copyright (C) 2013 nico3000 - All Rights Reserved
 */
package graphics.texture;

import java.util.LinkedList;
import java.util.Queue;
import logging.Logger;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL20;

/**
 * Verwaltet die verfuegbaren Textureinheiten.
 * @author nico3000
 */
public class SlotAdmin {
    /**
     * Liste aller Textureinheiten.
     */
    private final Queue<Integer> freeSlots = new LinkedList<>();
    
    /**
     * Konstruktor. Fuehrt potentiell fehlerwerfenden Code aus.
     */
    private SlotAdmin() {
        int maxSlots = GL11.glGetInteger(GL20.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS);
        // slot 0 is WORKING_SLOT!
        for(int i=1; i < maxSlots; ++i){
            this.freeSlots.offer(i);
        }
    }
    
    /**
     * Liefert eine freie Textureinheit und markiert diese als besetzt, bis sie
     * mit <tt>offerFreeSlot()</tt> wieder freigegeben wird.
     * @return Freie Textureinheit
     */
    public int getFreeSlot() {
        if(this.freeSlots.isEmpty()) {
            Logger.INSTANCE.error("no more free slots");
            return -1;
        } else {
            return this.freeSlots.poll();
        }
    }
    
    /**
     * Gibt eine Textureinheit wieder frei.
     * @param i Die freizugebende Einheit
     */
    public void offerFreeSlot(int i) {
        this.freeSlots.offer(i);
    }
    
    /**
     * Statische Singleton Methode.
     * @return Die Instanz
     */
    public static SlotAdmin getInstance() {
        return SlotAdminHolder.INSTANCE;
    }
    
    /**
     * Instanzhalterklasse fuer eine kontrollierte Erzeugung.
     */
    private static class SlotAdminHolder {
        /**
         * Die Instanz.
         */
        private static final SlotAdmin INSTANCE = new SlotAdmin();
    }
}
