/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gui;

import graphics.texture.Texture;
import graphics.texture.TextureLoader;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.font.GlyphVector;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import logging.Logger;
import main.GameApp;
import org.lwjgl.BufferUtils;
import resource.ResHandle;
import resource.Resource;
import resource.loader.ImageExtraData;

/**
 *
 * @author nico3000
 */
public class DefaultComponentDrawer implements ComponentDrawer {
    private Color DEFAULT_COLOR_BACKGROUND = new Color(0.0f, 0.0f, 0.0f, 0.8f);
    private Color HOVERED_COLOR_BACKGROUND = new Color(0.5f, 0.5f, 0.5f, 1.0f);
    private Color DEFAULT_COLOR_BORDER = new Color(1.0f, 1.0f, 1.0f, 1.0f);
    private Color HOVERED_COLOR_BORDER = new Color(1.0f, 1.0f, 1.0f, 1.0f);
    private Color DEFAULT_COLOR_FONT = new Color(1.0f, 1.0f, 1.0f, 1.0f);
    private Font DEFAULT_FONT = new Font("Arial", Font.PLAIN, 32);
    private static final int BORDER_LR = 4;
    private static final int BORDER_TB = 4;    
    
    protected BufferedImage createBaseImage(int width, int height) {
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics2D graphics = image.createGraphics();
        graphics.setBackground(new Color(1.0f, 1.0f, 1.0f, 0.0f));
        graphics.clearRect(0, 0, image.getWidth(), image.getHeight());
        return image;
    } 

    @Override
    public boolean init() {
        DEFAULT_COLOR_BACKGROUND = GameApp.getConfig().getAsColor("cDefaultColorBackground", DEFAULT_COLOR_BACKGROUND);
        HOVERED_COLOR_BACKGROUND = GameApp.getConfig().getAsColor("cHoveredColorBackground", HOVERED_COLOR_BACKGROUND);
        DEFAULT_COLOR_BORDER = GameApp.getConfig().getAsColor("cDefaultColorBorder", DEFAULT_COLOR_BORDER);
        HOVERED_COLOR_BORDER = GameApp.getConfig().getAsColor("cHoveredColorBorder", HOVERED_COLOR_BORDER);
        DEFAULT_COLOR_FONT = GameApp.getConfig().getAsColor("cDefaultColorFont", DEFAULT_COLOR_FONT);
        DEFAULT_FONT = new Font(GameApp.getConfig().getAsString("sFontName"), Font.PLAIN, 32);
        return true;
    }

    @Override
    public Texture createLabel(String text, int width, int height) {
        BufferedImage image = this.createBaseLabel(width, height);
        this.drawCenteredText(image, text, 0.0, 0.0);
        return TextureLoader.createTexture(image);
    }

    @Override
    public Texture createButton(String text, int width, int height, boolean focus, boolean hovered, boolean pressed) {
        BufferedImage image = this.createBaseButton(width, height, focus, hovered, pressed);
        this.drawCenteredText(image, text, pressed ? 1.0 : 0.0, pressed ? 1.0 : 0.0);
        return TextureLoader.createTexture(image);
    }

    @Override
    public Texture createImageLabel(String imageResource, int width, int height) {
        BufferedImage image = this.createBaseLabel(width, height);
        this.drawCenteredImage(image, imageResource, 0, 0);
        return TextureLoader.createTexture(image);
    }

    @Override
    public Texture createImageButton(String imageResource, int width, int height, boolean focus, boolean hovered, boolean pressed) {
        BufferedImage image = this.createBaseButton(width, height, focus, hovered, pressed);
        this.drawCenteredImage(image, imageResource, pressed ? 1 : 0, pressed ? 1 : 0);
        return TextureLoader.createTexture(image);
    }
    
    private BufferedImage createBaseLabel(int width, int height) {
        BufferedImage image = this.createBaseImage(width, height);
        Graphics2D graphics = image.createGraphics();
        graphics.setColor(DEFAULT_COLOR_BACKGROUND);
        graphics.fillRoundRect(0, 0, image.getWidth() - 1, image.getHeight() - 1, 2 * BORDER_LR, 2 * BORDER_TB);
        graphics.setColor(DEFAULT_COLOR_BORDER);
        graphics.drawRoundRect(0, 0, image.getWidth() - 1, image.getHeight() - 1, 2 * BORDER_LR, 2 * BORDER_TB);
        return image;
    }
    
    private BufferedImage createBaseButton(int width, int height, boolean focus, boolean hovered, boolean pressed) {
        BufferedImage image = this.createBaseImage(width, height);
        Graphics2D graphics = image.createGraphics();
        graphics.setColor(DEFAULT_COLOR_BACKGROUND);
        graphics.fill3DRect(0, 0, width - 1, height - 1, !pressed);
        //graphics.setColor(DEFAULT_COLOR_BORDER);
        //graphics.draw3DRect(0, 0, width - 1, height - 1, !pressed);
        return image;
    }
    
    private void drawCenteredText(BufferedImage image, String text, double offsetX, double offsetY) {
        if(text != null) {
            Graphics2D graphics = image.createGraphics();
            int size = 32;
            int maxWidth = image.getWidth() - 2 * BORDER_LR;
            int maxHeight = image.getHeight() - 2 * BORDER_TB;
            Font font;
            Font derived = DEFAULT_FONT;
            Rectangle2D rect;
            GlyphVector vec;
            do {
                font = derived;
                graphics.setFont(font);
                vec = font.createGlyphVector(graphics.getFontRenderContext(), text);
                rect = vec.getVisualBounds();
                derived = font.deriveFont((float)(size -= 2));
            } while(size > 2 && (rect.getWidth() > maxWidth || rect.getHeight() > maxHeight));
            //Logger.INSTANCE.info("Eventually size " + font.getSize() + " chosen.");
            graphics.setColor(DEFAULT_COLOR_FONT);
            graphics.drawGlyphVector(vec,
                    (float)(-rect.getX() + 0.5 * ((double)image.getWidth() - rect.getWidth()) + offsetX),
                    (float)(-rect.getY() + 0.5 * ((double)image.getHeight() - rect.getHeight()) + offsetY));
        }
    }   
    
    private void drawCenteredImage(BufferedImage target, String imageResource, int offsetX, int offsetY) {
        ResHandle handle = GameApp.getResCache().getHandle(new Resource(imageResource));
        if(handle == null) {
            Logger.INSTANCE.error(imageResource + " is not known!");
        } else {
            if(handle.getExtra() == null || !(handle.getExtra() instanceof ImageExtraData)) {
                Logger.INSTANCE.error(imageResource + " is no readable image!");
            }
            ImageExtraData extra = (ImageExtraData)handle.getExtra();

            FloatBuffer buffer = BufferUtils.createFloatBuffer(handle.getSize() / 4);
            ByteBuffer srcBuffer = ByteBuffer.wrap(handle.getBuffer());
            int max = extra.getWidth() * extra.getHeight() * extra.getChannels();
            for(int i=0; i < max; ++i) {
                buffer.put(srcBuffer.getFloat());
            }
            buffer.position(0);
            handle.unlock();
            BufferedImage image = new BufferedImage(extra.getWidth(), extra.getHeight(), BufferedImage.TYPE_4BYTE_ABGR);
            for(int y=0; y < extra.getHeight(); ++y) {
                for(int x=0; x < extra.getWidth(); ++x) {
                    float color[] = { 1.0f, 1.0f, 1.0f, 1.0f, };
                    for(int c=0; c < extra.getChannels(); ++c) {
                        color[c] = buffer.get();
                    }
                    image.setRGB(x, y, new Color(color[0], color[1], color[2], color[3]).getRGB());
                }
            }
            target.createGraphics().drawImage(image, BORDER_LR + offsetX, BORDER_TB + offsetY, target.getWidth() - 2 * BORDER_LR - 1, target.getHeight() - 2 * BORDER_TB - 1, null);
        }
    }

    @Override
    public Texture createTextField(String text, int caretPosition, int fontSize, int width, int height) {
        BufferedImage image = this.createBaseLabel(width, height);
        image.createGraphics();
        Graphics2D graphics = image.createGraphics();
        graphics.setColor(DEFAULT_COLOR_FONT);
        Font font = DEFAULT_FONT.deriveFont((float)fontSize);
        graphics.setFont(font);
        GlyphVector vec = font.createGlyphVector(graphics.getFontRenderContext(), text);
        Rectangle2D rect = vec.getVisualBounds();
        float x = (float)BORDER_LR;
        float y = (float)(-rect.getY() + 0.5 * ((double)image.getHeight() - rect.getHeight()));
        graphics.drawGlyphVector(vec, x, y);
        graphics.setColor(Color.BLACK);
        if(caretPosition != -1) {
            if(caretPosition < vec.getNumGlyphs()) {
                Rectangle bounds = vec.getGlyphPixelBounds(caretPosition, graphics.getFontRenderContext(), x, y);
                graphics.drawLine(bounds.x - 1, (int)y + 2, bounds.x - 1, (int)(y - rect.getHeight()) - 2);
            } else if(vec.getNumGlyphs() != 0) {
                Rectangle bounds = vec.getGlyphPixelBounds(caretPosition - 1, graphics.getFontRenderContext(), x, y);
                graphics.drawLine(bounds.x + bounds.width + 1, (int)y + 2, bounds.x + bounds.width + 1, (int)(y - rect.getHeight()) - 2);
            } else {
                graphics.drawLine(BORDER_LR, BORDER_TB + 2, BORDER_LR, BORDER_TB + fontSize + 2);
            }
        }
        return TextureLoader.createTexture(image);
    }
}
