/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gui.components;

/**
 * Ein Label ist eine Komponente, die nur Text oder ein Bild anzeigt und
 * ansonsten keine Funktionalitaet bietet.
 * @author nico3000
 */
public class Label extends Component {
    /**
     * Der Text des Labels.
     */
    private String text;

    /**
     * Konstruktor.
     * @param text Der Text des Labels
     */
    public Label(String text) {
        this.setText(text);
    }

    /**
     * Setzt den Text des Labels.
     * @param text Der neue Text
     */
    public final void setText(String text) {
        this.text = text;
        this.repaint();
    }

    /**
     * Liefert den aktuellen Text.
     * @return Der aktuelle Text
     */
    public String getText() {
        return text;
    }
    
    @Override
    protected void repaint() {
        if(this.getWidth() != 0 && this.getHeight() != 0) {
            if(this.text != null && this.text.startsWith("res:")) {
                this.setTexture(this.getGui().getDrawer().createImageLabel(this.text.substring(4), this.getWidth(), this.getHeight()));
            } else {
                this.setTexture(this.getGui().getDrawer().createLabel(this.text, this.getWidth(), this.getHeight()));
            }
        }
    }

    @Override
    public boolean onCursorEntered(int x, int y) {
        return true;
    }

    @Override
    public boolean onFocusGained() {
        return false;
    }

    @Override
    public boolean onDragged(int button, int x, int y, int dx, int dy) {
        return false;
    }

    @Override
    public void onFocusLost() {
        
    }

    @Override
    public void onCursorLeft() {
        // nothing to do
    }

    @Override
    public void onUpdate(long deltaMillis) {
        // nothing to do
    }

    @Override
    public void onCursorMoved(int posX, int posY, int dx, int dy) {
        // nothing to do
    }

    @Override
    public boolean onButtonDown(int button, int x, int y) {
        return true;
    }

    @Override
    public boolean onButtonUp(int button, int x, int y, boolean clicked) {
        return true;
    }

    @Override
    public boolean onKeyDown(int key, char c) {
        return false;
    }

    @Override
    public boolean onKeyUp(int key) {
        return false;
    }
    
}
