/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package input;

/**
 * Interface fuer eine Klasse, die auf Tastenanschlaege der Tastatur reagieren
 * kann.
 * @author nico3000
 */
public interface KeyboardHandler {
    /**
     * Wird aufgerufen, wenn eine Taste gedrueckt wird.
     * @param key Der Keycode der Taste
     * @param c Der zugehoerige Character, falls vorhanden; sonst <tt>0</tt>
     * @return <tt>true</tt>, falls das Loslassen verarbeitet wurde; false, wenn
     *         es ignoriert wurde und der naechste Handler benachrichtigt werden
     *         soll
     */
    public boolean onKeyDown(int key, char c);
    
    /**
     * Wird aufgerufen, wenn eine Taste losgelassen wird.
     * @param key Der Keycode der Taste
     * @return <tt>true</tt>, falls das Loslassen verarbeitet wurde; false, wenn
     *         es ignoriert wurde und der naechste Handler benachrichtigt werden
     *         soll
     */
    public boolean onKeyUp(int key);
    
    /**
     * Soll die Methode <tt>onKeyDown()</tt> auch bei Repeatevents aufgerufen
     * werden, muss diese Methode <tt>true</tt> zurueckliefern. Repeatevents
     * werden geworfen, falls eine Taste auf der Tastatur laenger gehalten wird.
     * @return <tt>true</tt>, falls <tt>onKeyDown()</tt> auch bei Repeatevents
     *         aufgerufen werden soll; <tt>false</tt> sonst
     */
    public boolean isProcessingRepeatEvents();
}
